@@ -15,6 +15,7 @@
 #include <limits.h>
 #include <stdint.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include "utf8.h"
 #include "util.h"
@@ -127,7 +128,7 @@ LIB_EXPORT void l_free(void *ptr)
  * l_strdup:
  * @str: string pointer
  *
- * Allocates and duplicates sring
+ * Allocates and duplicates string
  *
  * Returns: a newly allocated string
  **/
@@ -153,7 +154,7 @@ LIB_EXPORT char *l_strdup(const char *str)
  * @str: string pointer
  * @max: Maximum number of characters to copy
  *
- * Allocates and duplicates sring.  If the string is longer than @max
+ * Allocates and duplicates string.  If the string is longer than @max
  * characters, only @max are copied and a null terminating character
  * is added.
  *
@@ -812,3 +813,14 @@ LIB_EXPORT int l_safe_atox32(const char *s, uint32_t *out_x)
 
 	return safe_atou(s, 16, out_x);
 }
+
+LIB_EXPORT size_t l_util_pagesize(void)
+{
+	static size_t page_size = 0;
+
+	if (likely(page_size > 0))
+		return page_size;
+
+	page_size = sysconf(_SC_PAGESIZE);
+	return page_size;
+}
